import processing.sound.*;

Character character;
Object[] objects;
Enemy[] enemies;
MovingEnemy[] moving_enemies;
int timer;
float health_percentage;
boolean game_over;
boolean started_game = false;
PFont f;
int finishline_y;
SoundFile theme;
SoundFile pick;
SoundFile hit;
PImage hogwarts;

void setup() {
  size(1000, 720);
  f = createFont("ARCADE.TTF", 72);
  textFont(f);
  theme = new SoundFile(this, "rise-and-shine.mp3");
  pick = new SoundFile(this, "object pick.wav");
  hit = new SoundFile(this, "enemy hit.wav");
  character = new Character(500, 500, 57, "Main Character.png", 100, 140, 4);
  objects = new Object[30];
  enemies = new Enemy[30];
  moving_enemies = new MovingEnemy[30];
  hogwarts = loadImage("hogwarts.png");

  //object initialisation
  for (int i = 0; i<10; i++) {
    int choice = int(random(3)); //so that one of the three options is chosen
    objects[i] = new Object(random(275, 725), random(-5200, 695), 25, choice==0? "potion.png": choice==1? "charm.png": "chocolate bar.png", 50, choice==2? 87: 50); //so that as the negative is updated to positive, the illusion of movement is created

    //to ensure the objects don't overlap with each other
    for (int j = 0; j<i; j++) {
      if (i!=j && objects[i].distance(objects[j]) < 50) {
        while (objects[i].distance(objects[j]) < 50) {
          objects[i] = new Object(random(275, 725), random(-5200, 695), 25, choice==0? "potion.png": choice==1? "charm.png": "chocolate bar.png", 50, choice==2? 87: 50);
        }
      }
    }
  }

  //enemy initialisation
  for (int i = 0; i<30; i++) {
    enemies[i] = new Enemy(random(275, 725), random(-5200, 695), 30, "magic blast.png", 60, 48);

    //to ensure they don't overlap with each other
    for (int j = 0; j<i; j++) {
      if (i!=j && enemies[i].distance(enemies[j]) < 60) {
        while (enemies[i].distance(enemies[j]) < 60) {
          enemies[i] = new Enemy(random(275, 725), random(-5200, 695), 30, "magic blast.png", 60, 48);
        }
      }
    }
  }

  //moving enemies initialisation
  for (int i = 0; i<20; i++) {
    moving_enemies[i] = new MovingEnemy(random(280, 720), random(-12000, 690), 60, "dementor.png", 120, 116, 280, 720);

    //to ensure they don't overlap with each other
    for (int j = 0; j<i; j++) {
      if (i!=j && moving_enemies[i].distance(moving_enemies[j]) < 120) {
        while (moving_enemies[i].distance(moving_enemies[j]) < 120) {
          moving_enemies[i] = new MovingEnemy(random(280, 720), random(-12000, 690), 60, "dementor.png", 120, 116, 280, 720);
        }
      }
    }
  }
  timer = 65;
  health_percentage = 100;
  finishline_y = -2600;
  game_over = false;
  if (started_game == false)
    startScreen();
}

void draw() {

  if (started_game && !game_over) {
    if (frameCount % 60 == 0)
      timer-=1;    
    finishline_y+=1; //so that the finish is slowly reached
    gameDisplay();
  }
}

void startScreen() {
  background(0);
  textSize(60);
  text("MAGIC FIGHTER", 280, 300);
  textSize(30);
  text("Reach the safety of the castle in time!", 200, 365);
  text("Avoid the green magic and Dementors. Pick up the other objects!", 50, 400);
  textSize(25);
  text("Press the left and right arrow keys for movement", 250, 450);
  textSize(40);
  text("Click to begin!", 370, 490);
}

void gameDisplay() {
  
  //game winning conditions
  if (character.y == finishline_y) {
    background(0);
    fill(255);
    textSize(60);
    text("Congratulations!", 275, 300);
    text("You are safe now!", 240, 365);
    textSize(30);
    text("Click to play again", 365, 435);
    theme.stop();
    game_over = true;
    return;
  }
  
  //game losing conditions
  if (character.health <= 0 || timer <= 0) {
    background(0);
    fill(255, 255, 255);
    textSize(100);
    text("Oops, Game Over!", 145, 310);
    textSize(30);
    text("Click to play again", 365, 415);
    theme.stop();
    game_over = true;
    return;
  }

  //basic background
  background(#5D575F);
  noStroke();
  fill(#2E793B);
  rect(0, 0, 250, 720);
  fill(0);
  rect(750, 0, 1000, 720);

  //displaying the objects and enemies and character
  for (int i = 0; i<30; i++) {
    //picking  up objects
    if (objects[i]!=null) {
      if (dist(character.x, character.y, objects[i].x, objects[i].y) <= character.r + objects[i].r) {
        objects[i] = null;
        if (character.health+2 < 150) {
          character.health+=2;
        } else {
          while (character.health < 150) {
            character.health+=1;
          }
        }
        //object sounds
        pick.play();
      }
    }

    //being hit by enemies
    if (enemies[i]!=null) {
      if (dist(character.x, character.y, enemies[i].x, enemies[i].y) <= character.r/2 + enemies[i].r) {
        character.health -= 10;
        timer -= 1;
        enemies[i] = null;
        //hit sounds
        hit.play();
      }
    }

    if (moving_enemies[i]!=null) {
      if (dist(character.x, character.y, moving_enemies[i].x, moving_enemies[i].y) <= character.r/2 + moving_enemies[i].r) {
        character.health -= 10;
        timer -= 1;
        moving_enemies[i] = null;
        //hit sounds
        hit.play();
      }
    }

    //"finish line"
    imageMode(CORNER);
    image(hogwarts, 150, finishline_y-529);
    imageMode(CENTER);

    if (objects[i]!=null)
      objects[i].display();

    if (enemies[i]!=null)
      enemies[i].display();

    if (moving_enemies[i]!=null)
      moving_enemies[i].display();
  }

  //calculating health
  fill(255, 255, 255);
  textSize(36);
  text("Health: ", 780, 100);
  health_percentage = int((character.health * 100) / 150.0 * 100) / 100.0;
  noStroke();

  //To change colour depending on health
  if (health_percentage >= 50)
    fill(0, 250, 0);
  else if (health_percentage >= 20)
    fill(255, 255, 0);
  else
    fill(255, 0, 0);
  rect(780, 125, character.health, 25);
  noFill();
  strokeWeight(5);
  stroke(255, 255, 255);
  rect(780, 125, 150, 25);
  fill(255, 255, 255);
  textSize(22);
  text(str(health_percentage) + "%", 935, 150);

  //To display timer
  fill(255);
  textSize(36);
  text("Time: ", 780, 225);
  textSize(40);
  text(str(timer), 780, 260);
  
  character.display();
}

void mouseClicked() {
  if (started_game == false) {
    started_game = true; //to start game
    theme.play();
  } else if (game_over) {
    frameCount = -1; //to restart game
    theme.play();
  }
}

//for movement when the left or right keys are pressed
void keyPressed() {
  if (keyCode == LEFT)
    character.pressed_left = true;
  else if (keyCode == RIGHT)
    character.pressed_right = true;
}
void keyReleased() {
  if (keyCode == LEFT)
    character.pressed_left = false;
  else if (keyCode == RIGHT)
    character.pressed_right = false;
}
